require("town.nut");

class TestGoal extends GoalController 
{
	constructor()
	{
		GoalLog.Info("Welcome to a simple Goal Script. Towns will grow on my instructions.");
	}

	towns = null;
	last_date = 0;
}

function TestGoal::Start()
{
	local now = GoalDate.GetSystemTime();

	GoalGameSettings.SetValue("economy.town_growth_rate", 2);
	GoalGameSettings.SetValue("economy.fund_buildings", 0);
	this.last_date = GoalDate.GetCurrentDate();

	// Create the towns list
	GoalLog.Info("Create town list ... (can take a while on large maps)");
	this.towns = this.CreateTownList();

	// Set initial time the towns will grow
	foreach (town in this.towns) {
		town.rate = GoalBase.RandRange(20) + 10;
		town.CalculateGrowTime();
		town.SetGoal();
	}

	GoalLog.Info("Game setup done");
	GoalLog.Info("");

	/* Wait 30 seconds till game starts */
	while (GoalDate.GetSystemTime() - now < 10) {
		GoalLog.Info("Game starts in " + (10 - (GoalDate.GetSystemTime() - now)) + " seconds ...");
		this.Sleep(30);
	}
	GoalGame.Unpause();

	GoalLog.Info("Happy playing");

	while (true) {
		this.TownsGrow();
		this.Sleep(60);
	}
}

function TestGoal::CreateTownList()
{
	local result = [];

	/* Make a squirrel array of all towns.
	 * For each town, an instance of Town is created
	 * to store the data related to that town
	 */
	local all_towns = GoalTownList();
	foreach(t, _ in all_towns)
	{
		result.append(Town(t));
	}

	return result;
}

function TestGoal::TownsGrow()
{
	local date = GoalDate.GetCurrentDate();
	local diff = date - this.last_date;
	this.last_date = date;

	if (diff == 0) return;

	foreach (town in this.towns) {
//		if (!this.HasGoal(town)) continue;

		town.rate -= diff;

		if (town.rate <= 0) {
			town.rate = town.rate + GoalBase.RandRange(20) + 10;
			town.CalculateGrowTime();
			town.SetGoal();

//			GoalTown.ExpandTown(town.id, 1);
		}
	}
}
